# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import json
import logging
import os
from azure.synapse.artifacts.models import (LinkedService, Dataset, PipelineResource, RunFilterParameters,
                                            Trigger, DataFlow, BigDataPoolReference, NotebookSessionProperties,
                                            NotebookResource, SparkJobDefinition, SqlScriptResource, SqlScriptFolder,
                                            SqlScriptContent, SqlScriptMetadata, SqlScript, SqlConnection,
                                            NotebookFolder, LinkConnectionResource, LinkTableRequest,
                                            QueryTableStatusRequest, SecureString)
from azure.cli.core.util import sdk_no_wait, CLIError
from azure.core.exceptions import ResourceNotFoundError
from .._client_factory import (cf_synapse_linked_service, cf_synapse_dataset, cf_synapse_pipeline,
                               cf_synapse_pipeline_run, cf_synapse_trigger, cf_synapse_trigger_run,
                               cf_synapse_data_flow, cf_synapse_notebook, cf_synapse_spark_pool,
                               cf_synapse_spark_job_definition, cf_synapse_library, cf_synapse_sql_script,
                               cf_synapse_link_connection)
from ..constant import EXECUTOR_SIZE, SPARK_SERVICE_ENDPOINT_API_VERSION


# Linked Service
def list_linked_service(cmd, workspace_name):
    client = cf_synapse_linked_service(cmd.cli_ctx, workspace_name)
    return client.get_linked_services_by_workspace()


def get_linked_service(cmd, workspace_name, linked_service_name):
    client = cf_synapse_linked_service(cmd.cli_ctx, workspace_name)
    return client.get_linked_service(linked_service_name)


def create_or_update_linked_service(cmd, workspace_name, linked_service_name, definition_file, no_wait=False):
    client = cf_synapse_linked_service(cmd.cli_ctx, workspace_name)
    properties = LinkedService.from_dict(definition_file['properties'])
    return sdk_no_wait(no_wait, client.begin_create_or_update_linked_service,
                       linked_service_name, properties, polling=True)


def delete_linked_service(cmd, workspace_name, linked_service_name, no_wait=False):
    client = cf_synapse_linked_service(cmd.cli_ctx, workspace_name)
    return sdk_no_wait(no_wait, client.begin_delete_linked_service, linked_service_name, polling=True)


# Dataset
def list_datasets(cmd, workspace_name):
    client = cf_synapse_dataset(cmd.cli_ctx, workspace_name)
    return client.get_datasets_by_workspace()


def get_dataset(cmd, workspace_name, dataset_name):
    client = cf_synapse_dataset(cmd.cli_ctx, workspace_name)
    return client.get_dataset(dataset_name)


def create_or_update_dataset(cmd, workspace_name, dataset_name, definition_file, no_wait=False):
    client = cf_synapse_dataset(cmd.cli_ctx, workspace_name)
    properties = Dataset.from_dict(definition_file['properties'])
    return sdk_no_wait(no_wait, client.begin_create_or_update_dataset,
                       dataset_name, properties, polling=True)


def delete_dataset(cmd, workspace_name, dataset_name, no_wait=False):
    client = cf_synapse_dataset(cmd.cli_ctx, workspace_name)
    return sdk_no_wait(no_wait, client.begin_delete_dataset, dataset_name, polling=True)


# Pipeline
def list_pipelines(cmd, workspace_name):
    client = cf_synapse_pipeline(cmd.cli_ctx, workspace_name)
    return client.get_pipelines_by_workspace()


def get_pipeline(cmd, workspace_name, pipeline_name):
    client = cf_synapse_pipeline(cmd.cli_ctx, workspace_name)
    return client.get_pipeline(pipeline_name)


def create_or_update_pipeline(cmd, workspace_name, pipeline_name, definition_file, no_wait=False):
    client = cf_synapse_pipeline(cmd.cli_ctx, workspace_name)
    properties = PipelineResource.from_dict(definition_file['properties'])
    return sdk_no_wait(no_wait, client.begin_create_or_update_pipeline,
                       pipeline_name, properties, polling=True)


def delete_pipeline(cmd, workspace_name, pipeline_name, no_wait=False):
    client = cf_synapse_pipeline(cmd.cli_ctx, workspace_name)
    return sdk_no_wait(no_wait, client.begin_delete_pipeline, pipeline_name, polling=True)


def create_pipeline_run(cmd, workspace_name, pipeline_name, reference_pipeline_run_id=None, is_recovery=None,
                        start_activity_name=None, parameters=None):
    client = cf_synapse_pipeline(cmd.cli_ctx, workspace_name)
    return client.create_pipeline_run(pipeline_name, reference_pipeline_run_id, is_recovery,
                                      start_activity_name, parameters)


# Pipline run
def query_pipeline_runs_by_workspace(cmd, workspace_name, last_updated_after, last_updated_before,
                                     continuation_token=None, filters=None, order_by=None):
    client = cf_synapse_pipeline_run(cmd.cli_ctx, workspace_name)
    return client.query_pipeline_runs_by_workspace(RunFilterParameters(last_updated_after=last_updated_after,
                                                                       last_updated_before=last_updated_before,
                                                                       continuation_token=continuation_token,
                                                                       filters=filters, order_by=order_by))


def get_pipeline_run(cmd, workspace_name, run_id):
    client = cf_synapse_pipeline_run(cmd.cli_ctx, workspace_name)
    return client.get_pipeline_run(run_id)


def query_activity_runs(cmd, workspace_name, pipeline_name, run_id, last_updated_after, last_updated_before,
                        continuation_token=None, filters=None, order_by=None):
    client = cf_synapse_pipeline_run(cmd.cli_ctx, workspace_name)
    return client.query_activity_runs(pipeline_name, run_id,
                                      RunFilterParameters(last_updated_after=last_updated_after,
                                                          last_updated_before=last_updated_before,
                                                          continuation_token=continuation_token,
                                                          filters=filters, order_by=order_by))


def cancel_pipeline_run(cmd, workspace_name, run_id, is_recursive=None):
    client = cf_synapse_pipeline_run(cmd.cli_ctx, workspace_name)
    return client.cancel_pipeline_run(run_id, is_recursive)


# Trigger
def list_triggers(cmd, workspace_name):
    client = cf_synapse_trigger(cmd.cli_ctx, workspace_name)
    return client.get_triggers_by_workspace()


def get_trigger(cmd, workspace_name, trigger_name):
    client = cf_synapse_trigger(cmd.cli_ctx, workspace_name)
    return client.get_trigger(trigger_name)


def create_or_update_trigger(cmd, workspace_name, trigger_name, definition_file, no_wait=False):
    client = cf_synapse_trigger(cmd.cli_ctx, workspace_name)
    properties = Trigger.from_dict(definition_file['properties'])
    return sdk_no_wait(no_wait, client.begin_create_or_update_trigger,
                       trigger_name, properties, polling=True)


def delete_trigger(cmd, workspace_name, trigger_name, no_wait=False):
    client = cf_synapse_trigger(cmd.cli_ctx, workspace_name)
    return sdk_no_wait(no_wait, client.begin_delete_trigger, trigger_name, polling=True)


def subscribe_trigger_to_events(cmd, workspace_name, trigger_name, no_wait=False):
    client = cf_synapse_trigger(cmd.cli_ctx, workspace_name)
    return sdk_no_wait(no_wait, client.begin_subscribe_trigger_to_events, trigger_name, polling=True)


def get_event_subscription_status(cmd, workspace_name, trigger_name):
    client = cf_synapse_trigger(cmd.cli_ctx, workspace_name)
    return client.get_event_subscription_status(trigger_name)


def unsubscribe_trigger_from_events(cmd, workspace_name, trigger_name, no_wait=False):
    client = cf_synapse_trigger(cmd.cli_ctx, workspace_name)
    return sdk_no_wait(no_wait, client.begin_unsubscribe_trigger_from_events, trigger_name, polling=True)


def start_trigger(cmd, workspace_name, trigger_name, no_wait=False):
    client = cf_synapse_trigger(cmd.cli_ctx, workspace_name)
    return sdk_no_wait(no_wait, client.begin_start_trigger, trigger_name, polling=True)


def stop_trigger(cmd, workspace_name, trigger_name, no_wait=False):
    client = cf_synapse_trigger(cmd.cli_ctx, workspace_name)
    return sdk_no_wait(no_wait, client.begin_stop_trigger, trigger_name, polling=True)


# Trigger run
def rerun_trigger(cmd, workspace_name, trigger_name, run_id):
    client = cf_synapse_trigger_run(cmd.cli_ctx, workspace_name)
    return client.rerun_trigger_instance(trigger_name, run_id)


def cancel_trigger(cmd, workspace_name, trigger_name, run_id):
    client = cf_synapse_trigger_run(cmd.cli_ctx, workspace_name)
    return client.cancel_trigger_instance(trigger_name, run_id)


def query_trigger_runs_by_workspace(cmd, workspace_name, last_updated_after, last_updated_before,
                                    continuation_token=None, filters=None, order_by=None):
    client = cf_synapse_trigger_run(cmd.cli_ctx, workspace_name)
    return client.query_trigger_runs_by_workspace(RunFilterParameters(last_updated_after=last_updated_after,
                                                                      last_updated_before=last_updated_before,
                                                                      continuation_token=continuation_token,
                                                                      filters=filters, order_by=order_by))


# Data flow
def list_data_flows(cmd, workspace_name):
    client = cf_synapse_data_flow(cmd.cli_ctx, workspace_name)
    return client.get_data_flows_by_workspace()


def get_data_flow(cmd, workspace_name, data_flow_name):
    client = cf_synapse_data_flow(cmd.cli_ctx, workspace_name)
    return client.get_data_flow(data_flow_name)


def create_or_update_data_flow(cmd, workspace_name, data_flow_name, definition_file, no_wait=False):
    client = cf_synapse_data_flow(cmd.cli_ctx, workspace_name)
    properties = DataFlow.from_dict(definition_file['properties'])
    return sdk_no_wait(no_wait, client.begin_create_or_update_data_flow,
                       data_flow_name, properties, polling=True)


def delete_data_flow(cmd, workspace_name, data_flow_name, no_wait=False):
    client = cf_synapse_data_flow(cmd.cli_ctx, workspace_name)
    return sdk_no_wait(no_wait, client.begin_delete_data_flow, data_flow_name, polling=True)


# Notebook
def create_or_update_notebook(cmd, workspace_name, definition_file, notebook_name, spark_pool_name=None,
                              folder_path=None, executor_size="Small", executor_count=2, no_wait=False):
    client = cf_synapse_notebook(cmd.cli_ctx, workspace_name)
    spark_pool_client = cf_synapse_spark_pool(cmd.cli_ctx, workspace_name)
    if spark_pool_name is not None:
        endpoint = '{}{}{}'.format("https://", workspace_name, cmd.cli_ctx.cloud.suffixes.synapse_analytics_endpoint)
        spark_pool_info = spark_pool_client.get(spark_pool_name)
        metadata = definition_file['metadata']
        options = {}
        options['auth'] = {'type': 'AAD',
                           'authResource': '{}{}'.format("https://",
                                                         cmd.cli_ctx.cloud.suffixes.synapse_analytics_endpoint[1:])}
        options['cores'] = EXECUTOR_SIZE[executor_size]['Cores']
        options['memory'] = EXECUTOR_SIZE[executor_size]['Memory']
        options['nodeCount'] = executor_count
        options['endpoint'] = '{}{}{}{}{}'.format(endpoint, '/livyApi/versions/', SPARK_SERVICE_ENDPOINT_API_VERSION,
                                                  '/sparkPools/', spark_pool_name)
        options['extraHeader'] = {}
        options['id'] = spark_pool_info.id
        options['name'] = spark_pool_name
        options['sparkVersion'] = spark_pool_info.spark_version
        options['type'] = 'Spark'
        metadata['a365ComputeOptions'] = options

        definition_file['bigDataPool'] = BigDataPoolReference(type='BigDataPoolReference',
                                                              reference_name=spark_pool_name)
        definition_file['sessionProperties'] = NotebookSessionProperties(driver_memory=options['memory'],
                                                                         driver_cores=options['cores'],
                                                                         executor_memory=options['memory'],
                                                                         executor_cores=options['cores'],
                                                                         num_executors=executor_count)
    definition_file['folder'] = NotebookFolder(name=folder_path)
    properties = NotebookResource(name=notebook_name, properties=definition_file)
    return sdk_no_wait(no_wait, client.begin_create_or_update_notebook,
                       notebook_name, properties, polling=True)


def list_notebooks(cmd, workspace_name):
    client = cf_synapse_notebook(cmd.cli_ctx, workspace_name)
    return client.get_notebooks_by_workspace()


def get_notebook(cmd, workspace_name, notebook_name):
    client = cf_synapse_notebook(cmd.cli_ctx, workspace_name)
    return client.get_notebook(notebook_name)


def export_notebook(cmd, workspace_name, output_folder, notebook_name=None):
    client = cf_synapse_notebook(cmd.cli_ctx, workspace_name)
    if notebook_name is not None:
        notebook = client.get_notebook(notebook_name)
        path = os.path.join(output_folder, notebook.name + '.ipynb')
        print(notebook.properties.as_dict())
        write_to_file(notebook, path)
    else:
        notebooks = client.get_notebooks_by_workspace()
        for notebook in notebooks:
            path = os.path.join(output_folder, notebook.name + '.ipynb')
            print(notebook.properties.as_dict())
            write_to_file(notebook, path)


def metadata_processing(notebook_properties, displayedWidgets):
    synapseWidgetNotebookMetadataVersion = '0.1'
    metadata = {}
    notebook_properties_metadata = {}
    for key in list(notebook_properties.keys()):
        if key == 'metadata':
            notebook_properties_metadata = notebook_properties['metadata']

    if notebook_properties_metadata is None:
        return metadata, displayedWidgets

    for elementkey in list(notebook_properties_metadata.keys()):
        if elementkey == 'language_info':
            if notebook_properties_metadata['language_info'] and \
                    'codemirror_mode' in notebook_properties_metadata['language_info']:
                notebook_properties_metadata['language_info'].pop('codemirror_mode')
            metadata['language_info'] = notebook_properties_metadata['language_info']
        elif elementkey == 'description':
            metadata['description'] = notebook_properties_metadata['description']
        elif elementkey == 'saveOutput':
            metadata['save_output'] = notebook_properties_metadata['saveOutput']
        elif elementkey == 'kernelspec':
            metadata['kernelspec'] = notebook_properties_metadata['kernelspec']
        elif elementkey == 'synapse_widget' and \
                'state' in notebook_properties_metadata['synapse_widget']:
            for ekey in list(notebook_properties_metadata['synapse_widget']['state'].keys()):
                for i in reversed(range(len(displayedWidgets))):
                    if displayedWidgets[i]['widget_id'] == ekey:
                        displayedWidgets.pop(i)
            metadata['synapse_widget'] = notebook_properties_metadata['synapse_widget']
            metadata['synapse_widget']['version'] = synapseWidgetNotebookMetadataVersion
    return metadata, displayedWidgets


def write_to_file(notebook, path):
    try:
        notebook_properties = notebook.properties.as_dict()
        livyStatementMetaOutputContentType = 'application/vnd.livy.statement-meta+json'
        synapseWidgetViewOutputContentType = 'application/vnd.synapse.widget-view+json'
        notebook_result = {}
        displayedWidgets = []
        notebook_result['nbformat'] = 4
        notebook_result['nbformat_minor'] = 2
        for cell in notebook_properties['cells']:
            if cell['cell_type'] == 'code' and cell['outputs']:
                for output in cell['outputs']:
                    if output['output_type'] == 'display_data' and \
                            synapseWidgetViewOutputContentType in output['data']:
                        displayedWidgets.append(output["data"]["application/vnd.synapse.widget-view+json"])

        metadata_results, displayedWidgets_results = \
            metadata_processing(notebook_properties, displayedWidgets)

        if len(displayedWidgets_results) > 0:
            logging.info('Detected widget with missing data.')

        for cell in notebook_properties['cells']:
            if cell['cell_type'] == 'code' and cell['outputs']:
                for output in cell['outputs']:
                    if output['output_type'] == 'display_data' and output['data'] and \
                            livyStatementMetaOutputContentType in output['data']:
                        output['data'].pop(livyStatementMetaOutputContentType)

        notebook_result['metadata'] = metadata_results
        notebook_result['cells'] = notebook_properties['cells']
        with open(path, 'w') as f:
            json.dump(notebook_result, f, indent=4)
    except IOError:
        raise CLIError('Unable to export to file: {}'.format(path))


def delete_notebook(cmd, workspace_name, notebook_name, no_wait=False):
    client = cf_synapse_notebook(cmd.cli_ctx, workspace_name)
    return sdk_no_wait(no_wait, client.begin_delete_notebook, notebook_name, polling=True)


# Workspace package
def list_workspace_package(cmd, workspace_name):
    client = cf_synapse_library(cmd.cli_ctx, workspace_name)
    return client.list()


def get_workspace_package(cmd, workspace_name, package_name):
    client = cf_synapse_library(cmd.cli_ctx, workspace_name)
    return client.get(package_name)


def upload_workspace_package(cmd, workspace_name, package, progress_callback=None):
    client = cf_synapse_library(cmd.cli_ctx, workspace_name)
    package_name = os.path.basename(package)

    # Check if the package already exists
    if test_workspace_package(client, package_name):
        raise CLIError("A workspace package with name '{0}' already exists.".format(
                       package_name))

    # Create package
    client.begin_create(package_name).result()
    # Upload package content
    package_size = os.path.getsize(package)
    chunk_size = 4 * 1024 * 1024
    if progress_callback is not None:
        progress_callback(0, package_size)
    with open(package, 'rb') as stream:
        index = 0
        while True:
            read_size = min(chunk_size, package_size - index)
            data = stream.read(read_size)

            if data == b'':
                break

            client.append(package_name, data)
            index += len(data)
            if progress_callback is not None:
                progress_callback(index, package_size)
    # Call Flush API as a completion signal
    client.begin_flush(package_name).result()

    return client.get(package_name)


def workspace_package_upload_batch(cmd, workspace_name, source, progress_callback=None):
    # Tell progress reporter to reuse the same hook
    if progress_callback:
        progress_callback.reuse = True

    source_files = []
    results = []
    for root, _, files in os.walk(source):
        for f in files:
            full_path = os.path.join(root, f)
            source_files.append(full_path)

    for index, source_file in enumerate(source_files):
        # add package name and number to progress message
        if progress_callback:
            progress_callback.message = '{}/{}: "{}"'.format(
                index + 1, len(source_files), os.path.basename(source_file))

        results.append(upload_workspace_package(cmd, workspace_name, source_file, progress_callback))

    # end progress hook
    if progress_callback:
        progress_callback.hook.end()

    return results


def test_workspace_package(client, package_name):
    try:
        client.get(package_name)
        return True
    except ResourceNotFoundError:
        return False


def delete_workspace_package(cmd, workspace_name, package_name, no_wait=False):
    client = cf_synapse_library(cmd.cli_ctx, workspace_name)
    return sdk_no_wait(no_wait, client.begin_delete, package_name)


# Spark job definition
def list_spark_job_definition(cmd, workspace_name):
    client = cf_synapse_spark_job_definition(cmd.cli_ctx, workspace_name)
    return client.get_spark_job_definitions_by_workspace()


def get_spark_job_definition(cmd, workspace_name, spark_job_definition_name):
    client = cf_synapse_spark_job_definition(cmd.cli_ctx, workspace_name)
    return client.get_spark_job_definition(spark_job_definition_name)


def delete_spark_job_definition(cmd, workspace_name, spark_job_definition_name, no_wait=False):
    client = cf_synapse_spark_job_definition(cmd.cli_ctx, workspace_name)
    return sdk_no_wait(no_wait, client.begin_delete_spark_job_definition, spark_job_definition_name, polling=True)


def create_or_update_spark_job_definition(cmd, workspace_name, spark_job_definition_name, definition_file,
                                          folder_path=None, no_wait=False):
    client = cf_synapse_spark_job_definition(cmd.cli_ctx, workspace_name)
    folder = {}
    folder['name'] = folder_path
    definition_file['folder'] = folder
    properties = SparkJobDefinition.from_dict(definition_file)
    return sdk_no_wait(no_wait, client.begin_create_or_update_spark_job_definition,
                       spark_job_definition_name, properties, polling=True)


def list_sql_scripts(cmd, workspace_name):
    client = cf_synapse_sql_script(cmd.cli_ctx, workspace_name)
    return client.get_sql_scripts_by_workspace()


def get_sql_script(cmd, workspace_name, sql_script_name):
    client = cf_synapse_sql_script(cmd.cli_ctx, workspace_name)
    return client.get_sql_script(sql_script_name)


def delete_sql_script(cmd, workspace_name, sql_script_name, no_wait=False):
    client = cf_synapse_sql_script(cmd.cli_ctx, workspace_name)
    return sdk_no_wait(no_wait, client.begin_delete_sql_script, sql_script_name, polling=True)


def create_sql_script(cmd, workspace_name, sql_script_name, definition_file, result_limit=5000,
                      folder_name=None, description=None, sql_pool_name=None,
                      sql_database_name=None, additional_properties=None, no_wait=False):
    client = cf_synapse_sql_script(cmd.cli_ctx, workspace_name)
    try:
        with open(definition_file, 'r') as stream:
            query = stream.read()
    except:
        from azure.cli.core.azclierror import InvalidArgumentValueError
        err_msg = 'Definition file path is invalid'
        raise InvalidArgumentValueError(err_msg)
    if sql_pool_name:
        if sql_pool_name.lower() == 'built-in':
            sql_pool_type = 'SqlOnDemand'
            current_connection = SqlConnection(type=sql_pool_type,
                                               pool_name='Built-in',
                                               database_name=sql_database_name)
        else:
            sql_pool_type = 'SqlPool'
            current_connection = SqlConnection(type=sql_pool_type,
                                               pool_name=sql_pool_name,
                                               database_name=sql_database_name)
    else:
        current_connection = SqlConnection(type='SqlOnDemand',
                                           pool_name='Built-in',
                                           database_name='master')

    sql_script_content = SqlScriptContent(query=query,
                                          current_connection=current_connection,
                                          result_limit=result_limit,
                                          metadata=SqlScriptMetadata(language='sql'))
    sql_script_folder = SqlScriptFolder(name=folder_name)
    properties = SqlScript(additional_properties=additional_properties,
                           description=description,
                           type='SqlQuery',
                           content=sql_script_content,
                           folder=sql_script_folder)
    sql_script = SqlScriptResource(name=sql_script_name, properties=properties)
    return sdk_no_wait(no_wait, client.begin_create_or_update_sql_script,
                       sql_script_name, sql_script, polling=True)


def export_sql_script(cmd, workspace_name, output_folder, sql_script_name=None):
    client = cf_synapse_sql_script(cmd.cli_ctx, workspace_name)
    if sql_script_name is not None:
        sql_script_query = client.get_sql_script(sql_script_name).properties.content.query
        path = os.path.join(output_folder, sql_script_name + '.sql')
        try:
            with open(path, 'w') as f:
                f.write(sql_script_query)
            print(sql_script_name + 'export success')
        except:
            from azure.cli.core.azclierror import InvalidArgumentValueError
            err_msg = 'Unable to export to file: {}'.format(path)
            raise InvalidArgumentValueError(err_msg)
    else:
        sql_scripts = client.get_sql_scripts_by_workspace()
        for sql_script in sql_scripts:
            sql_script_query = client.get_sql_script(sql_script.name).properties.content.query
            path = os.path.join(output_folder, sql_script.name + '.sql')
            try:
                with open(path, 'w') as f:
                    f.write(sql_script_query)
                print(sql_script.name + 'export success')
            except:
                from azure.cli.core.azclierror import InvalidArgumentValueError
                err_msg = 'Unable to export to file: {}'.format(path)
                raise InvalidArgumentValueError(err_msg)


def list_link_connection(cmd, workspace_name):
    client = cf_synapse_link_connection(cmd.cli_ctx, workspace_name)
    return client.list_link_connections_by_workspace()


def get_link_connection(cmd, workspace_name, link_connection_name):
    client = cf_synapse_link_connection(cmd.cli_ctx, workspace_name)
    return client.get_link_connection(link_connection_name)


def create_or_update_link_connection(cmd, workspace_name, link_connection_name, definition_file):
    client = cf_synapse_link_connection(cmd.cli_ctx, workspace_name)
    info = definition_file['properties']
    properties_file = {}
    properties_file['sourceDatabase'] = info['sourceDatabase']
    properties_file['targetDatabase'] = info['targetDatabase']
    properties_file['compute'] = info['compute']
    if 'landingZone' in properties_file:
        properties_file['landingZone'] = info['landingZone']
    properties = LinkConnectionResource(properties=properties_file)
    return client.create_or_update_link_connection(link_connection_name, properties)


def delete_link_connection(cmd, workspace_name, link_connection_name):
    client = cf_synapse_link_connection(cmd.cli_ctx, workspace_name)
    return client.delete_link_connection(link_connection_name)


def get_link_connection_status(cmd, workspace_name, link_connection_name):
    client = cf_synapse_link_connection(cmd.cli_ctx, workspace_name)
    return client.get_detailed_status(link_connection_name)


def start_link_connection(cmd, workspace_name, link_connection_name):
    client = cf_synapse_link_connection(cmd.cli_ctx, workspace_name)
    return client.start(link_connection_name)


def stop_link_connection(cmd, workspace_name, link_connection_name):
    client = cf_synapse_link_connection(cmd.cli_ctx, workspace_name)
    return client.stop(link_connection_name)


def synapse_list_link_table(cmd, workspace_name, link_connection_name):
    client = cf_synapse_link_connection(cmd.cli_ctx, workspace_name)
    return client.list_link_tables(link_connection_name).value


def synapse_edit_link_table(cmd, workspace_name, link_connection_name, definition_file):
    client = cf_synapse_link_connection(cmd.cli_ctx, workspace_name)
    linkTableRequset_list = []
    for i in range(0, len(definition_file['linkTables'])):
        linkTableRequset = LinkTableRequest.from_dict(definition_file['linkTables'][i])
        linkTableRequset_list.append(linkTableRequset)
    return client.edit_tables(link_connection_name, linkTableRequset_list)


def synapse_get_link_tables_status(cmd, workspace_name, link_connection_name, max_segment_count=50,
                                   continuation_token=None):
    client = cf_synapse_link_connection(cmd.cli_ctx, workspace_name)
    query_table_status_request = QueryTableStatusRequest(
        max_segment_count=max_segment_count,
        continuation_token=continuation_token
    )
    return client.query_table_status(link_connection_name, query_table_status_request)


def synapse_update_landing_zone_credential(cmd, workspace_name, link_connection_name, sas_token):
    client = cf_synapse_link_connection(cmd.cli_ctx, workspace_name)
    sas_tokens = SecureString(value=sas_token)
    return client.update_landing_zone_credential(link_connection_name, sas_tokens)



# pylint: disable=line-too-long
# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=too-many-statements


from knack.arguments import CLIArgumentType
from knack.log import get_logger
from azure.mgmt.signalr.models import SignalRRequestType
from azure.cli.core.commands.validators import get_default_location_from_resource_group
from azure.cli.core.commands.parameters import (
    resource_group_name_type,
    get_location_type,
    get_resource_name_completion_list,
    tags_type,
    get_enum_type,
    get_three_state_flag
)
from ._actions import (
    UpstreamTemplateAddAction
)
from ._constants import (
    SIGNALR_RESOURCE_TYPE,
    SIGNALR_KEY_TYPE,
    SIGNALR_SERVICE_MODE_TYPE
)


logger = get_logger(__name__)


def load_arguments(self, _):
    signalr_name_type = CLIArgumentType(options_list='--signalr-name', help='Name of the SignalR.', id_part='name')
    signalr_custom_domain_name_type = CLIArgumentType(help='Name of the custom domain.', id_part='child_name_1')
    signalr_custom_certificate_name_type = CLIArgumentType(help='Name of the custom certificate.', id_part='child_name_2')

    with self.argument_context('signalr') as c:
        c.argument('resource_group_name', arg_type=resource_group_name_type)
        c.argument('location',
                   arg_type=get_location_type(self.cli_ctx),
                   validator=get_default_location_from_resource_group)
        c.argument('signalr_name', signalr_name_type, options_list=['--name', '-n'],
                   completer=get_resource_name_completion_list(SIGNALR_RESOURCE_TYPE),
                   help='Name of signalr service.')
        c.argument('tags', arg_type=tags_type)

    with self.argument_context('signalr create') as c:
        c.argument('sku', help='The sku name of the signalr service. Allowed values: Premium_P1, Standard_S1, Free_F1')
        c.argument('unit_count', help='The number of signalr service unit count', type=int)
        c.argument('service_mode', help='The service mode which signalr service will be working on', choices=SIGNALR_SERVICE_MODE_TYPE)
        c.argument('enable_message_logs', help='The switch for messaging logs which signalr service will generate or not', arg_type=get_three_state_flag())
        c.argument('allowed_origins', options_list=['--allowed-origins', '-a'], nargs='*', help='space separated origins that should be allowed to make cross-origin calls (for example: http://example.com:12345). To allow all, use "*"')

    with self.argument_context('signalr update') as c:
        c.argument('sku', help='The sku name of the signalr service. E.g. Standard_S1')
        c.argument('unit_count', help='The number of signalr service unit count', type=int)
        c.argument('service_mode', help='The service mode which signalr service will be working on', choices=SIGNALR_SERVICE_MODE_TYPE)
        c.argument('enable_message_logs', help='The switch for messaging logs which signalr service will generate or not', arg_type=get_three_state_flag())
        c.argument('allowed_origins', options_list=['--allowed-origins', '-a'], nargs='*', help='space separated origins that should be allowed to make cross-origin calls (for example: http://example.com:12345). To allow all, use "*"')

    for scope in ['signalr create', 'signalr update']:
        with self.argument_context(scope, arg_group='Network Rule') as c:
            c.argument('default_action', arg_type=get_enum_type(['Allow', 'Deny']), help='Default action to apply when no rule matches.', required=False)

    with self.argument_context('signalr key renew') as c:
        c.argument('key_type', help='The name of access key to regenerate', choices=SIGNALR_KEY_TYPE)

    with self.argument_context('signalr key list') as c:
        c.argument('signalr_name', id_part=None)

    with self.argument_context('signalr cors add') as c:
        c.argument('allowed_origins', options_list=['--allowed-origins', '-a'], nargs='*', help='space separated origins that should be allowed to make cross-origin calls (for example: http://example.com:12345). To allow all, use "*"')

    with self.argument_context('signalr cors remove') as c:
        c.argument('allowed_origins', options_list=['--allowed-origins', '-a'], nargs='*', help='space separated origins that should be allowed to make cross-origin calls (for example: http://example.com:12345). To allow all, use "*"')

    with self.argument_context('signalr cors update') as c:
        c.argument('allowed_origins', options_list=['--allowed-origins', '-a'], nargs='*', help='space separated origins that should be allowed to make cross-origin calls (for example: http://example.com:12345). To allow all, use "*"')

    with self.argument_context('signalr cors list') as c:
        c.argument('signalr_name', id_part=None)

    # Network Rule
    with self.argument_context('signalr network-rule update') as c:
        c.argument('connection_name', nargs='*', help='Space-separeted list of private endpoint connection name.', required=False, arg_group='Private Endpoint Connection')
        c.argument('public_network', arg_type=get_three_state_flag(), help='Set rules for public network.', required=False, arg_group='Public Network')
        c.argument('allow', nargs='*', help='The allowed virtual network rule. Space-separeted list of scope to assign. Allowed values: ClientConnection, ServerConnection, RESTAPI', type=SignalRRequestType, required=False)
        c.argument('deny', nargs='*', help='The denied virtual network rule. Space-separeted list of scope to assign. Allowed values: ClientConnection, ServerConnection, RESTAPI', type=SignalRRequestType, required=False)

    with self.argument_context('signalr network-rule list') as c:
        c.argument('signalr_name', id_part=None)

    # Upstream Settings
    with self.argument_context('signalr upstream update') as c:
        c.argument('template', action=UpstreamTemplateAddAction, nargs='+', help='Template item for upstream settings. Use key=value pattern to set properties. Supported keys are "url-template", "hub-pattern", "event-pattern", "category-pattern".')

    with self.argument_context('signalr upstream list') as c:
        c.argument('signalr_name', id_part=None)

    # Managed Identity
    with self.argument_context('signalr identity assign') as c:
        c.argument('identity', help="Assigns managed identities to the service. Use '[system]' to refer to the system-assigned identity or a resource ID to refer to a user-assigned identity. You can only assign either on of them.")

    # Custom Domain
    for scope in ['signalr custom-domain update',
                  'signalr custom-domain create',
                  'signalr custom-domain show',
                  'signalr custom-domain delete',
                  'signalr custom-domain list']:
        with self.argument_context(scope) as c:
            c.argument('signalr_name', signalr_name_type, id_part=None)
            c.argument('name', signalr_custom_domain_name_type)

    for scope in ['signalr custom-domain update',
                  'signalr custom-domain create']:
        with self.argument_context(scope) as c:
            c.argument('domain_name', help="Custom domain name. For example, `contoso.com`.")
            c.argument('certificate_resource_id', help="ResourceId of a previously created custom certificate.")

    # Custom Certificate
    for scope in ['signalr custom-certificate update',
                  'signalr custom-certificate create',
                  'signalr custom-certificate show',
                  'signalr custom-certificate delete',
                  'signalr custom-certificate list']:
        with self.argument_context(scope) as c:
            c.argument('signalr_name', signalr_name_type, id_part=None)
            c.argument('name', signalr_custom_certificate_name_type)

    for scope in ['signalr custom-certificate update',
                  'signalr custom-certificate create']:
        with self.argument_context(scope) as c:
            c.argument('keyvault_base_uri', help="Key vault base URI. For example, `https://contoso.vault.azure.net`.")
            c.argument('keyvault_secret_name', help="Key vault secret name where certificate is stored.")
            c.argument('keyvault_secret_version', help="Key vault secret version where certificate is stored. If empty, will use latest version.")



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


def cf_resource_groups(cli_ctx, subscription_id=None):
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    from azure.cli.core.profiles import ResourceType
    return get_mgmt_service_client(cli_ctx, ResourceType.MGMT_RESOURCE_RESOURCES,
                                   subscription_id=subscription_id).resource_groups


def cf_storage(cli_ctx, *_, **__):
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    from azure.mgmt.storage import StorageManagementClient
    return get_mgmt_service_client(cli_ctx, StorageManagementClient)


def cf_network(cli_ctx, aux_subscriptions=None, **_):
    from azure.cli.core.profiles import ResourceType
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    return get_mgmt_service_client(cli_ctx, ResourceType.MGMT_NETWORK,
                                   aux_subscriptions=aux_subscriptions)


def cf_graph(cli_ctx, **_):
    from azure.cli.command_modules.role import graph_client_factory
    client = graph_client_factory(cli_ctx)
    return client


def cf_resources(cli_ctx, **_):
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    from azure.cli.core.profiles import ResourceType
    return get_mgmt_service_client(cli_ctx, ResourceType.MGMT_RESOURCE_RESOURCES)


def cf_log_analytics(cli_ctx, *_, **__):
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    from azure.mgmt.loganalytics import LogAnalyticsManagementClient
    return get_mgmt_service_client(cli_ctx, LogAnalyticsManagementClient)


def cf_hdinsight(cli_ctx, *_, **__):
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    from azure.mgmt.hdinsight import HDInsightManagementClient
    return get_mgmt_service_client(cli_ctx, HDInsightManagementClient)


def cf_hdinsight_clusters(cli_ctx, *_, **__):
    return cf_hdinsight(cli_ctx).clusters


def cf_hdinsight_script_actions(cli_ctx, *_, **__):
    return cf_hdinsight(cli_ctx).script_actions


def cf_hdinsight_script_execution_history(cli_ctx, *_, **__):
    return cf_hdinsight(cli_ctx).script_execution_history


def cf_hdinsight_applications(cli_ctx, *_, **__):
    return cf_hdinsight(cli_ctx).applications


def cf_hdinsight_extensions(cli_ctx, *_, **__):
    return cf_hdinsight(cli_ctx).extensions


def cf_hdinsight_locations(cli_ctx, *_, **__):
    return cf_hdinsight(cli_ctx).locations


def cf_hdinsight_virtual_machines(cli_ctx, *_, **__):
    return cf_hdinsight(cli_ctx).virtual_machines



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import uuid
from knack.log import get_logger
from msrestazure.tools import parse_resource_id, is_valid_resource_id

logger = get_logger(__name__)

# region Definitions Custom Commands


# pylint: disable=unused-argument
def cli_definition_create(cmd, client,
                          name, tenant_id, principal_id, role_definition_id,
                          plan_name=None, plan_product=None, plan_publisher=None, plan_version=None, description=None,
                          definition_id=None):
    from azure.mgmt.managedservices.models import RegistrationDefinitionProperties, Plan, \
        Authorization

    if not definition_id:
        definition_id = str(uuid.uuid4())

    subscription = _get_subscription_id_from_cmd(cmd)
    scope = _get_scope(subscription)

    if plan_name and plan_product and plan_publisher and plan_version:
        plan = Plan(plan_name, plan_publisher, plan_product, plan_version)
    else:
        plan = None

    authorization = Authorization(
        principal_id=principal_id,
        role_definition_id=role_definition_id)

    authorizations = [authorization]

    properties = RegistrationDefinitionProperties(
        description=description,
        authorizations=authorizations,
        registration_definition_name=name,
        managed_by_tenant_id=tenant_id,
        Plan=plan,
    )

    return client.create_or_update(
        registration_definition_id=definition_id,
        scope=scope,
        properties=properties)


# pylint: disable=unused-argument
def cli_definition_get(cmd, client,
                       definition):
    subscription = _get_subscription_id_from_cmd(cmd)
    definition_id, sub_id, rg_name = _get_resource_id_parts(cmd, definition, subscription)
    scope = _get_scope(sub_id, rg_name)
    return client.get(
        scope=scope,
        registration_definition_id=definition_id)


# pylint: disable=unused-argument
def cli_definition_list(cmd, client):
    subscription = _get_subscription_id_from_cmd(cmd)
    scope = _get_scope(subscription)
    return client.list(scope=scope)


def cli_definition_delete(cmd, client, definition):
    subscription = _get_subscription_id_from_cmd(cmd)
    definition_id, sub_id, rg_name = _get_resource_id_parts(cmd, definition, subscription)
    scope = _get_scope(sub_id, rg_name)
    return client.delete(
        scope=scope,
        registration_definition_id=definition_id)


# endregion

# region Assignments Custom Commands

# pylint: disable=unused-argument
def cli_assignment_create(cmd, client,
                          definition,
                          assignment_id=None,
                          resource_group_name=None):
    from azure.mgmt.managedservices.models import RegistrationAssignmentProperties
    if not is_valid_resource_id(definition):
        raise ValueError(
            "definition should be a valid resource id. For example, "
            "/subscriptions/id/providers/Microsoft.ManagedServices/registrationDefinitions/id")

    if not assignment_id:
        assignment_id = str(uuid.uuid4())

    subscription = _get_subscription_id_from_cmd(cmd)
    scope = _get_scope(subscription, resource_group_name)
    properties = RegistrationAssignmentProperties(
        registration_definition_id=definition)
    return client.create_or_update(
        scope=scope,
        registration_assignment_id=assignment_id,
        properties=properties)


# pylint: disable=unused-argument
def cli_assignment_get(cmd, client,
                       assignment,
                       resource_group_name=None,
                       include_definition=None):
    subscription = _get_subscription_id_from_cmd(cmd)
    assignment_id, sub_id, rg_name = _get_resource_id_parts(cmd, assignment, subscription, resource_group_name)
    scope = _get_scope(sub_id, rg_name)
    return client.get(
        scope=scope,
        registration_assignment_id=assignment_id,
        expand_registration_definition=include_definition)


# pylint: disable=unused-argument
def cli_assignment_delete(cmd, client,
                          assignment,
                          resource_group_name=None):
    subscription = _get_subscription_id_from_cmd(cmd)
    assignment_id, sub_id, rg_name = _get_resource_id_parts(cmd, assignment, subscription, resource_group_name)
    scope = _get_scope(sub_id, rg_name)
    return client.delete(
        scope=scope,
        registration_assignment_id=assignment_id)


# pylint: disable=unused-argument
def cli_assignment_list(cmd, client,
                        resource_group_name=None,
                        include_definition=None):
    sub_id = _get_subscription_id_from_cmd(cmd)
    scope = _get_scope(sub_id, resource_group_name)
    return client.list(
        scope=scope,
        expand_registration_definition=include_definition)

# endregion

# region private methods


def _get_subscription_id_from_cmd(cmd):
    from azure.cli.core.commands.client_factory import get_subscription_id
    subscription = get_subscription_id(cmd.cli_ctx)
    return subscription


def _get_resource_id_parts(cmd, name_or_id, subscription=None, resource_group_name=None):
    rg_name = None
    if is_valid_resource_id(name_or_id):
        id_parts = parse_resource_id(name_or_id)
        sub_id = id_parts['subscription']
        name = id_parts['name']
        if 'resource_group' in id_parts:
            rg_name = id_parts['resource_group']
    else:
        rg_name = resource_group_name
        name = name_or_id
        sub_id = _get_subscription_id_from_cmd(cmd)
    return name, sub_id, rg_name


def _get_scope(subscription_id, resource_group_name=None):
    if not resource_group_name:
        scope = "/subscriptions/" + subscription_id
    else:
        scope = "/subscriptions/" + subscription_id + "/resourceGroups/" + resource_group_name

    return scope
# endregion



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=line-too-long
# pylint: disable=too-many-statements
# pylint: disable=too-many-locals


# pylint: disable=line-too-long
def load_arguments_eh(self, _):
    from azure.cli.core.commands.parameters import tags_type, get_enum_type, resource_group_name_type, name_type, \
        get_location_type, get_three_state_flag, get_resource_name_completion_list
    from azure.cli.core.commands.validators import get_default_location_from_resource_group
    from azure.cli.command_modules.eventhubs._completers import get_consumergroup_command_completion_list, \
        get_eventhubs_command_completion_list
    from azure.cli.command_modules.eventhubs._validator import validate_storageaccount, validate_partner_namespace, validate_rights
    from knack.arguments import CLIArgumentType
    from azure.cli.core.profiles import ResourceType
    (KeyType, AccessRights, SkuName, TlsVersion) = self.get_models('KeyType', 'AccessRights', 'SkuName', 'TlsVersion', resource_type=ResourceType.MGMT_EVENTHUB)
    from azure.cli.command_modules.eventhubs.action import AlertAddEncryption, ConstructPolicy

    rights_arg_type = CLIArgumentType(options_list=['--rights'], nargs='+', arg_type=get_enum_type(AccessRights), validator=validate_rights, help='Space-separated list of Authorization rule rights')
    key_arg_type = CLIArgumentType(options_list=['--key'], arg_type=get_enum_type(KeyType), help='specifies Primary or Secondary key needs to be reset')
    keyvalue_arg_type = CLIArgumentType(options_list=['--key-value'], help='Optional, if the key value provided, is set for KeyType or autogenerated Key value set for keyType.')
    event_hub_name_arg_type = CLIArgumentType(options_list=['--eventhub-name'], help='Name of EventHub')
    namespace_name_arg_type = CLIArgumentType(options_list=['--namespace-name'], help='Name of Namespace', id_part='name')

    with self.argument_context('eventhubs') as c:
        c.argument('resource_group_name', arg_type=resource_group_name_type)
        c.argument('namespace_name', id_part='name', help='name of Namespace')

    with self.argument_context('eventhubs namespace exists') as c:
        c.argument('name', arg_type=name_type, help='Namespace name. Name can contain only letters, numbers, and hyphens. The namespace must start with a letter, and it must end with a letter or number.')

    with self.argument_context('eventhubs namespace', min_api='2021-06-01-preview') as c:
        c.argument('namespace_name', arg_type=name_type, id_part='name', completer=get_resource_name_completion_list('Microsoft.ServiceBus/namespaces'), help='Name of Namespace')
        c.argument('is_kafka_enabled', options_list=['--enable-kafka'], arg_type=get_three_state_flag(),
                   help='A boolean value that indicates whether Kafka is enabled for eventhub namespace.')
        c.argument('tags', arg_type=tags_type)
        c.argument('sku', options_list=['--sku'], arg_type=get_enum_type(SkuName), help='Namespace SKU.')
        c.argument('location', arg_type=get_location_type(self.cli_ctx), validator=get_default_location_from_resource_group)
        c.argument('capacity', type=int, help='Capacity for Sku')
        c.argument('is_auto_inflate_enabled', options_list=['--enable-auto-inflate'], arg_type=get_three_state_flag(), help='A boolean value that indicates whether AutoInflate is enabled for eventhub namespace.')
        c.argument('maximum_throughput_units', type=int, help='Upper limit of throughput units when AutoInflate is enabled, vaule should be within 0 to 20 throughput units. ( 0 if AutoInflateEnabled = true)')
        c.argument('zone_redundant', options_list=['--zone-redundant'], is_preview=True, arg_type=get_three_state_flag(),
                   help='Enabling this property creates a Standard EventHubs Namespace in regions supported availability zones')
        c.argument('disable_local_auth', options_list=['--disable-local-auth'], is_preview=True, arg_type=get_three_state_flag(),
                   help='A boolean value that indicates whether SAS authentication is enabled/disabled for the Event Hubs')
        c.argument('mi_system_assigned', arg_group='Managed Identity',
                   arg_type=get_three_state_flag(),
                   help='Enable System Assigned Identity')
        c.argument('mi_user_assigned', arg_group='Managed Identity', nargs='+', help='List of User Assigned Identity ids.')
        c.argument('encryption_config', action=AlertAddEncryption, nargs='+', help='List of KeyVaultProperties objects.')
        c.argument('minimum_tls_version', arg_type=get_enum_type(TlsVersion), options_list=['--minimum-tls-version', '--min-tls'], help='The minimum TLS version for the cluster to support, e.g. 1.2')
        c.argument('require_infrastructure_encryption', options_list=['--infra-encryption'], is_preview=True,
                   arg_type=get_three_state_flag(),
                   help='A boolean value that indicates whether Infrastructure Encryption (Double Encryption) is enabled/disabled')

    with self.argument_context('eventhubs namespace create', min_api='2021-06-01-preview') as c:
        c.argument('cluster_arm_id', options_list=['--cluster-arm-id'], is_preview=True, help='Cluster ARM ID of the Namespace')


# Cluster region
    for scope in ['eventhubs cluster', 'eventhubs cluster namespace list']:
        with self.argument_context(scope) as c:
            c.argument('cluster_name', arg_type=name_type, id_part=None, help='Name of Cluster')

    with self.argument_context('eventhubs cluster create') as c:
        c.argument('location', arg_type=get_location_type(self.cli_ctx), id_part=None, help='Location of the Cluster, for locations of available pre-provision clusters, please check az event hubs ')
        c.argument('supports_scaling', arg_type=get_three_state_flag(), help='A value that indicates whether Scaling is Supported.')

    for scope in ['eventhubs cluster create', 'eventhubs cluster update']:
        with self.argument_context(scope) as c:
            c.argument('tags', arg_type=tags_type)
            c.argument('capacity', type=int, help='Capacity for Sku, allowed value : 1')

    # region Namespace Authorizationrule
    with self.argument_context('eventhubs namespace authorization-rule list') as c:
        c.argument('namespace_name', options_list=['--namespace-name'], id_part=None, help='Name of Namespace')

    with self.argument_context('eventhubs namespace authorization-rule keys list') as c:
        c.argument('authorization_rule_name', arg_type=name_type, id_part=None, help='Name of Namespace AuthorizationRule')
        c.argument('namespace_name', options_list=['--namespace-name'], id_part=None, help='Name of Namespace')

    for scope in ['eventhubs namespace authorization-rule', 'eventhubs namespace authorization-rule keys renew']:
        with self.argument_context(scope) as c:
            c.argument('authorization_rule_name', arg_type=name_type, id_part='child_name_1', help='Name of Namespace AuthorizationRule')
            c.argument('namespace_name', arg_type=namespace_name_arg_type, help='Name of Namespace')

    for scope in ['eventhubs namespace authorization-rule create', 'eventhubs namespace authorization-rule update', 'eventhubs eventhub authorization-rule create', 'eventhubs eventhub authorization-rule update']:
        with self.argument_context(scope) as c:
            c.argument('name', arg_type=name_type, help='Name of Authorization Rule')
            c.argument('rights', arg_type=rights_arg_type)

    with self.argument_context('eventhubs namespace authorization-rule keys renew') as c:
        c.argument('name', arg_type=name_type, help='Name of Authorization Rule')
        c.argument('key_type', arg_type=key_arg_type)
        c.argument('key', arg_type=keyvalue_arg_type)

# region - Eventhub Create
    with self.argument_context('eventhubs eventhub') as c:
        c.argument('event_hub_name', arg_type=name_type, id_part='child_name_1', completer=get_eventhubs_command_completion_list, help='Name of Eventhub')

    for scope in ['eventhubs eventhub update', 'eventhubs eventhub create']:
        with self.argument_context(scope) as c:
            c.argument('message_retention_in_days', options_list=['--message-retention'], type=int, help='Number of days to retain events for this Event Hub, value should be 1 to 7 days and depends on Namespace sku. if Namespace sku is Basic than value should be one and is Manadatory parameter. Namespace sku is standard value should be 1 to 7 days, default is 7 days and is optional parameter')
            c.argument('partition_count', type=int, help='Number of partitions created for the Event Hub. By default, allowed values are 2-32. Lower value of 1 is supported with Kafka enabled namespaces. In presence of a custom quota, the upper limit will match the upper limit of the quota.')
            c.argument('status', arg_type=get_enum_type(['Active', 'Disabled', 'SendDisabled']), help='Status of Eventhub')
            c.argument('enabled', options_list=['--enable-capture'], arg_type=get_three_state_flag(), help='A boolean value that indicates whether capture is enabled.')
            c.argument('skip_empty_archives', options_list=['--skip-empty-archives'], arg_type=get_three_state_flag(), help='A boolean value that indicates whether to Skip Empty.')
            c.argument('capture_interval_seconds', arg_group='Capture', options_list=['--capture-interval'], type=int, help='Allows you to set the frequency with which the capture to Azure Blobs will happen, value should between 60 to 900 seconds')
            c.argument('capture_size_limit_bytes', arg_group='Capture', options_list=['--capture-size-limit'], type=int, help='Defines the amount of data built up in your Event Hub before an capture operation, value should be between 10485760 to 524288000 bytes')
            c.argument('destination_name', arg_group='Capture-Destination', help='Name for capture destination, should be EventHubArchive.AzureBlockBlob.')
            c.argument('storage_account_resource_id', arg_group='Capture-Destination', validator=validate_storageaccount, options_list=['--storage-account'], help='Name (if within same resource group and not of type Classic Storage) or ARM id of the storage account to be used to create the blobs')
            c.argument('blob_container', arg_group='Capture-Destination', help='Blob container Name')
            c.argument('archive_name_format', arg_group='Capture-Destination', help='Blob naming convention for archive, e.g. {Namespace}/{EventHub}/{PartitionId}/{Year}/{Month}/{Day}/{Hour}/{Minute}/{Second}. Here all the parameters (Namespace,EventHub .. etc) are mandatory irrespective of order')

    with self.argument_context('eventhubs eventhub list') as c:
        c.argument('namespace_name', options_list=['--namespace-name'], id_part=None, help='Name of Namespace')

    # region EventHub Authorizationrule
    for scope in ['eventhubs eventhub authorization-rule']:
        with self.argument_context(scope) as c:
            c.argument('authorization_rule_name', arg_type=name_type, id_part='child_name_2', help='Name of EventHub AuthorizationRule')
            c.argument('event_hub_name', id_part='child_name_1', arg_type=event_hub_name_arg_type, help='Name of EventHub')

    with self.argument_context('eventhubs eventhub authorization-rule keys renew') as c:
        c.argument('name', arg_type=name_type, help='Name of Authorization Rule')
        c.argument('key_type', arg_type=key_arg_type)
        c.argument('key', arg_type=keyvalue_arg_type)

    with self.argument_context('eventhubs eventhub authorization-rule list') as c:
        c.argument('namespace_name', options_list=['--namespace-name'], id_part=None, help='Name of Namespace')
        c.argument('event_hub_name', id_part=None, arg_type=event_hub_name_arg_type)

    with self.argument_context('eventhubs eventhub authorization-rule keys list') as c:
        c.argument('namespace_name', options_list=['--namespace-name'], id_part=None, help='Name of Namespace')
        c.argument('event_hub_name', id_part=None, arg_type=event_hub_name_arg_type)
        c.argument('authorization_rule_name', arg_type=name_type, id_part=None, help='Name of EventHub AuthorizationRule')


# - ConsumerGroup Region
    with self.argument_context('eventhubs eventhub consumer-group') as c:
        c.argument('event_hub_name', arg_type=event_hub_name_arg_type, help='Name of EventHub')
        c.argument('consumer_group_name', arg_type=name_type, id_part='child_name_2', completer=get_consumergroup_command_completion_list, help='Name of ConsumerGroup')

    for scope in ['eventhubs eventhub consumer-group create', 'eventhubs eventhub consumer-group update']:
        with self.argument_context(scope) as c:
            c.argument('name', arg_type=name_type, help='Name of ConsumerGroup')
            c.argument('user_metadata', help='Usermetadata is a placeholder to store user-defined string data with maximum length 1024. e.g. it can be used to store descriptive data, such as list of teams and their contact information also user-defined configuration settings can be stored.')

    with self.argument_context('eventhubs eventhub consumer-group list') as c:
        c.argument('event_hub_name', arg_type=event_hub_name_arg_type, id_part=None, help='Name of EventHub')
        c.argument('namespace_name', options_list=['--namespace-name'], id_part=None, help='Name of Namespace')

#   : Region Geo DR Configuration
    with self.argument_context('eventhubs georecovery-alias') as c:
        c.argument('alias', options_list=['--alias', '-a'], id_part='child_name_1', help='Name of Geo-Disaster Recovery Configuration Alias')

    with self.argument_context('eventhubs georecovery-alias exists') as c:
        c.argument('name', options_list=['--alias', '-a'], arg_type=name_type, help='Name of Geo Recovery Configs - Alias to check availability')
        c.argument('namespace_name', options_list=['--namespace-name'], id_part=None, help='Name of Namespace')

    with self.argument_context('eventhubs georecovery-alias set') as c:
        c.argument('partner_namespace', required=True, validator=validate_partner_namespace, help='Name (if within the same resource group) or ARM Id of the Primary/Secondary eventhub namespace name, which is part of GEO DR pairing')
        c.argument('alternate_name', help='Alternate Name for the Alias, when the Namespace name and Alias name are same')

    for scope in ['eventhubs georecovery-alias authorization-rule show']:
        with self.argument_context(scope)as c:
            c.argument('authorization_rule_name', arg_type=name_type, id_part='child_name_2', help='Name of Namespace AuthorizationRule')

    with self.argument_context('eventhubs georecovery-alias list') as c:
        c.argument('namespace_name', options_list=['--namespace-name'], id_part=None, help='Name of Namespace')

    with self.argument_context('eventhubs georecovery-alias authorization-rule list') as c:
        c.argument('alias', options_list=['--alias', '-a'], help='Name of Geo-Disaster Recovery Configuration Alias')
        c.argument('namespace_name', options_list=['--namespace-name'], id_part=None, help='Name of Namespace')

    with self.argument_context('eventhubs georecovery-alias authorization-rule keys list') as c:
        c.argument('alias', options_list=['--alias', '-a'], id_part=None, help='Name of Geo-Disaster Recovery Configuration Alias')
        c.argument('namespace_name', options_list=['--namespace-name'], id_part=None, help='Name of Namespace')
        c.argument('authorization_rule_name', arg_type=name_type, help='Name of Namespace AuthorizationRule')

# Region Namespace NetworkRuleSet
    with self.argument_context('eventhubs namespace network-rule', resource_type=ResourceType.MGMT_EVENTHUB, min_api='2017-04-01') as c:
        c.argument('namespace_name', options_list=['--namespace-name'], id_part=None, help='Name of the Namespace')

    for scope in ['eventhubs namespace network-rule add', 'eventhubs namespace network-rule remove']:
        with self.argument_context(scope, resource_type=ResourceType.MGMT_EVENTHUB, min_api='2017-04-01') as c:
            c.argument('subnet', arg_group='Virtual Network Rule', options_list=['--subnet'], help='Name or ID of subnet. If name is supplied, `--vnet-name` must be supplied.')
            c.argument('ip_mask', arg_group='IP Address Rule', options_list=['--ip-address'], help='IPv4 address or CIDR range - 10.6.0.0/24')
            c.argument('namespace_name', options_list=['--namespace-name'], id_part=None, help='Name of the Namespace')
            c.extra('vnet_name', arg_group='Virtual Network Rule', options_list=['--vnet-name'], help='Name of the Virtual Network')

    with self.argument_context('eventhubs namespace network-rule update', resource_type=ResourceType.MGMT_EVENTHUB, min_api='2017-04-01') as c:
        c.argument('public_network_access', options_list=['--public-network-access', '--public-network'], arg_type=get_enum_type(['Enabled', 'Disabled']), help='This determines if traffic is allowed over public network. By default it is enabled. If value is SecuredByPerimeter then Inbound and Outbound communication is controlled by the network security perimeter and profile\' access rules.')
        c.argument('trusted_service_access_enabled', options_list=['--enable-trusted-service-access', '-t'],
                   arg_type=get_three_state_flag(),
                   help='A boolean value that indicates whether Trusted Service Access is enabled for Network Rule Set.')
        c.argument('default_action', arg_group='networkrule', options_list=['--default-action'],
                   arg_type=get_enum_type(['Allow', 'Deny']),
                   help='Default Action for Network Rule Set.')

    with self.argument_context('eventhubs namespace network-rule add', resource_type=ResourceType.MGMT_EVENTHUB, min_api='2017-04-01') as c:
        c.argument('ignore_missing_vnet_service_endpoint', arg_group='Virtual Network Rule', options_list=['--ignore-missing-endpoint'], arg_type=get_three_state_flag(), help='A boolean value that indicates whether to ignore missing vnet Service Endpoint')
        c.argument('action', arg_group='IP Address Rule', options_list=['--action'], arg_type=get_enum_type(['Allow']), help='Action of the IP rule')

# Private end point connection
    with self.argument_context('eventhubs namespace private-endpoint-connection',
                               resource_type=ResourceType.MGMT_EVENTHUB) as c:
        c.argument('namespace_name', options_list=['--namespace-name'], id_part=None, help='Name of the Namespace')
        c.argument('private_endpoint_connection_name', options_list=['--name', '-n'],
                   help='The name of the private endpoint connection associated with the EventHubs Namespace.')
    for item in ['approve', 'reject', 'show', 'delete']:
        with self.argument_context('eventhubs namespace private-endpoint-connection {}'.format(item),
                                   resource_type=ResourceType.MGMT_EVENTHUB) as c:
            c.argument('private_endpoint_connection_name', options_list=['--name', '-n'], required=False,
                       help='The name of the private endpoint connection associated with the EventHubs Namespace.')
            c.extra('connection_id', options_list=['--id'],
                    help='The ID of the private endpoint connection associated with the EventHubs Namespace. You can get '
                         'it using `az eventhubs namespace show`.')
            c.argument('namespace_name', help='The eventhubs namesapce name.', required=False)
            c.argument('resource_group_name', help='The resource group name of specified eventhubs namespace.',
                       required=False)
            c.argument('description', help='Comments for {} operation.'.format(item))

# Private end point connection
    with self.argument_context('eventhubs namespace private-link-resource',
                               resource_type=ResourceType.MGMT_EVENTHUB) as c:
        c.argument('namespace_name', options_list=['--namespace-name'], id_part=None, help='Name of the Namespace')

# Identity
    with self.argument_context('eventhubs namespace identity',
                               resource_type=ResourceType.MGMT_EVENTHUB) as c:
        c.argument('namespace_name', options_list=['--namespace-name'], id_part=None, help='Name of the Namespace')

    for scope in ['eventhubs namespace identity assign', 'eventhubs namespace identity remove']:
        with self.argument_context(scope, resource_type=ResourceType.MGMT_EVENTHUB) as c:
            c.argument('namespace_name', options_list=['--namespace-name'], id_part=None, help='Name of the Namespace')
            c.argument('system_assigned', arg_type=get_three_state_flag(), help='System Assigned Identity')
            c.argument('user_assigned', nargs='+', help='User Assigned Identity')

# Encryption
    with self.argument_context('eventhubs namespace encryption', resource_type=ResourceType.MGMT_EVENTHUB) as c:
        c.argument('namespace_name', options_list=['--namespace-name'], id_part=None, help='Name of the Namespace')

    for scope in ['eventhubs namespace encryption add', 'eventhubs namespace encryption remove']:
        with self.argument_context(scope, resource_type=ResourceType.MGMT_EVENTHUB) as c:
            c.argument('encryption_config', action=AlertAddEncryption, nargs='+', help='List of KeyVaultProperties objects.')
            c.argument('require_infrastructure_encryption', options_list=['--infra-encryption'], is_preview=True,
                       arg_type=get_three_state_flag(),
                       help='A boolean value that indicates whether Infrastructure Encryption (Double Encryption) is enabled/disabled')

# Schema Registry
    with self.argument_context('eventhubs namespace schema-registry list') as c:
        c.argument('namespace_name', options_list=['--namespace-name'], id_part=None, help='Name of Namespace')

    with self.argument_context('eventhubs namespace schema-registry') as c:
        c.argument('namespace_name', arg_type=namespace_name_arg_type, options_list=['--namespace-name'], help='name of Namespace')
        c.argument('schema_group_name', arg_type=name_type, id_part='child_name_1', help='name of schema group')

    for scope in ['eventhubs namespace schema-registry create', 'eventhubs namespace schema-registry update']:
        with self.argument_context(scope) as c:
            c.argument('schema_compatibility', options_list=['--schema-compatibility'], arg_type=get_enum_type(['None', 'Backward', 'Forward']), help='Compatibility of Schema')
            c.argument('schema_type', options_list=['--schema-type'], arg_type=get_enum_type(['Avro']), help='Type of Schema')
            c.argument('tags', options_list=['--group-properties'], arg_type=tags_type,
                       help='Type of Schema')

# Application Group
    with self.argument_context('eventhubs namespace application-group') as c:
        c.argument('namespace_name', options_list=['--namespace-name'], arg_type=namespace_name_arg_type, help='Name of Namespace')
        c.argument('application_group_name', arg_type=name_type, id_part='child_name_1', help='Name of Application Group')

    for scope in ['eventhubs namespace application-group create', 'eventhubs namespace application-group list']:
        with self.argument_context(scope) as c:
            c.argument('namespace_name', options_list=['--namespace-name'], id_part=None, help='Name of Namespace')
            c.argument('application_group_name', arg_type=name_type, id_part=None, help='Name of Application Group')

    for scope in ['eventhubs namespace application-group create', 'eventhubs namespace application-group update']:
        with self.argument_context(scope) as c:
            c.argument('is_enabled', arg_type=get_three_state_flag(),
                       help='Determines if Application Group is allowed to create connection with namespace or not. '
                            'Once the isEnabled is set to false, all the existing connections of application group gets dropped and no new connections will be allowed')

    with self.argument_context('eventhubs namespace application-group create') as c:
        c.argument('client_app_group_identifier', options_list=['--client-app-group-identifier', '--client-app-group-id'], help='The Unique identifier for application group.Supports SAS(SASKeyName=KeyName) or AAD(AADAppID=Guid)')

    for scope in ['eventhubs namespace application-group policy add', 'eventhubs namespace application-group policy remove', 'eventhubs namespace application-group create']:
        with self.argument_context(scope) as c:
            c.argument('throttling_policy_config', action=ConstructPolicy, options_list=['--throttling-policy-config', '--throttling-policy', '--policy-config'], nargs='+', help='List of Throttling Policy Objects')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.commands import CliCommandType
from ._client_factory import (get_mediaservices_client, get_transforms_client,
                              get_assets_client, get_jobs_client, get_streaming_locators_client,
                              get_streaming_policies_client, get_streaming_endpoints_client,
                              get_locations_client, get_live_events_client, get_live_outputs_client,
                              get_content_key_policies_client, get_asset_filters_client,
                              get_account_filters_client, get_asset_tracks_client)
from ._exception_handler import ams_exception_handler


# pylint: disable=line-too-long

def load_command_table(self, _):  # pylint: disable=too-many-locals, too-many-statements
    def get_sdk(operation, client_factory):
        return CliCommandType(
            operations_tmpl='azure.mgmt.media.operations#{}Operations.'.format(operation) + '{}',
            client_factory=client_factory,
            exception_handler=ams_exception_handler
        )

    def get_custom_sdk(custom_module, client_factory):
        return CliCommandType(
            operations_tmpl='azure.cli.command_modules.ams.operations.{}#'.format(custom_module) + '{}',
            client_factory=client_factory,
            exception_handler=ams_exception_handler
        )

    with self.command_group('ams account', get_sdk('Mediaservices', get_mediaservices_client)) as g:
        g.custom_show_command('show', 'get_mediaservice',
                              custom_command_type=get_custom_sdk('account', get_mediaservices_client))
        g.command('delete', 'delete')
        g.generic_update_command('update',
                                 getter_name='mediaservice_update_getter',
                                 getter_type=get_custom_sdk('account', get_mediaservices_client),
                                 custom_func_name='update_mediaservice',
                                 custom_func_type=get_custom_sdk('account', get_mediaservices_client))
        g.custom_command('list', 'list_mediaservices',
                         custom_command_type=get_custom_sdk('account', get_mediaservices_client))
        g.custom_command('create', 'create_mediaservice',
                         custom_command_type=get_custom_sdk('account', get_mediaservices_client))
        g.custom_command('check-name', 'check_name_availability',
                         custom_command_type=get_custom_sdk('account', get_locations_client))

    with self.command_group('ams account storage', get_sdk('Mediaservices', get_mediaservices_client)) as g:
        g.custom_command('add', 'add_mediaservice_secondary_storage',
                         custom_command_type=get_custom_sdk('account', get_mediaservices_client))
        g.custom_command('remove', 'remove_mediaservice_secondary_storage',
                         custom_command_type=get_custom_sdk('account', get_mediaservices_client))
        g.custom_command('set-authentication', 'set_mediaservice_trusted_storage',
                         custom_command_type=get_custom_sdk('account', get_mediaservices_client))
        g.custom_command('sync-storage-keys', 'sync_storage_keys',
                         custom_command_type=get_custom_sdk('account', get_mediaservices_client))

    with self.command_group('ams account sp', get_sdk('Mediaservices', get_mediaservices_client)) as g:
        g.custom_command('create', 'create_or_update_assign_sp_to_mediaservice',
                         custom_command_type=get_custom_sdk('sp', get_mediaservices_client))
        g.custom_command('reset-credentials', 'reset_sp_credentials_for_mediaservice',
                         custom_command_type=get_custom_sdk('sp', get_mediaservices_client))

    with self.command_group('ams account mru', get_sdk('Mediaservices', get_mediaservices_client)) as g:
        g.custom_show_command('show', 'get_mru',
                              custom_command_type=get_custom_sdk('mru', get_mediaservices_client))
        g.custom_command('set', 'set_mru',
                         custom_command_type=get_custom_sdk('mru', get_mediaservices_client))

    with self.command_group('ams account encryption', get_sdk('Mediaservices', get_mediaservices_client)) as g:
        g.custom_show_command('show', 'get_encryption',
                              custom_command_type=get_custom_sdk('encryption', get_mediaservices_client))
        g.custom_command('set', 'set_encryption',
                         custom_command_type=get_custom_sdk('encryption', get_mediaservices_client))

    with self.command_group('ams account identity', get_sdk('Mediaservices', get_mediaservices_client)) as g:
        g.custom_show_command('assign', 'assign_identity',
                              custom_command_type=get_custom_sdk('identity', get_mediaservices_client))
        g.custom_command('remove', 'remove_identity',
                         custom_command_type=get_custom_sdk('identity', get_mediaservices_client))
        g.custom_show_command('show', 'show_identity',
                              custom_command_type=get_custom_sdk('identity', get_mediaservices_client))

    with self.command_group('ams transform', get_sdk('Transforms', get_transforms_client)) as g:
        g.custom_show_command('show', 'get_transform',
                              custom_command_type=get_custom_sdk('transform', get_transforms_client))
        g.command('list', 'list')
        g.command('delete', 'delete')
        g.custom_command('create', 'create_transform',
                         custom_command_type=get_custom_sdk('transform', get_transforms_client))
        g.generic_update_command('update',
                                 setter_name='transform_update_setter',
                                 setter_type=get_custom_sdk('transform', get_mediaservices_client),
                                 custom_func_name='update_transform',
                                 custom_func_type=get_custom_sdk('transform', get_mediaservices_client))

    with self.command_group('ams transform output', get_sdk('Transforms', get_mediaservices_client)) as g:
        g.custom_command('add', 'add_transform_output',
                         custom_command_type=get_custom_sdk('transform', get_transforms_client))
        g.custom_command('remove', 'remove_transform_output',
                         custom_command_type=get_custom_sdk('transform', get_transforms_client))

    with self.command_group('ams asset', get_sdk('Assets', get_assets_client)) as g:
        g.custom_show_command('show', 'get_asset', custom_command_type=get_custom_sdk('asset', get_assets_client))
        g.command('list', 'list')
        g.command('delete', 'delete')
        g.command('list-streaming-locators', 'list_streaming_locators')
        g.custom_command('get-encryption-key', 'get_encryption_key',
                         custom_command_type=get_custom_sdk('asset', get_assets_client))
        g.generic_update_command('update',
                                 custom_func_name='update_asset',
                                 custom_func_type=get_custom_sdk('asset', get_mediaservices_client))
        g.custom_command('get-sas-urls', 'get_sas_urls',
                         custom_command_type=get_custom_sdk('asset', get_assets_client))
        g.custom_command('create', 'create_asset',
                         custom_command_type=get_custom_sdk('asset', get_assets_client))

    with self.command_group('ams asset-filter', get_sdk('AssetFilters', get_asset_filters_client)) as g:
        g.command('list', 'list')
        g.custom_show_command('show', 'get_asset_filter',
                              custom_command_type=get_custom_sdk('asset_filter', get_asset_filters_client))
        g.command('delete', 'delete')
        g.custom_command('create', 'create_asset_filter',
                         custom_command_type=get_custom_sdk('asset_filter', get_asset_filters_client))
        g.generic_update_command('update',
                                 custom_func_name='update_asset_filter',
                                 custom_func_type=get_custom_sdk('asset_filter', get_mediaservices_client))

    with self.command_group('ams asset-track', get_sdk('Tracks', get_asset_tracks_client)) as g:
        g.command('list', 'list')
        g.custom_show_command('show', 'get_asset_track',
                              custom_command_type=get_custom_sdk('tracks', get_asset_tracks_client))
        g.command('delete', 'begin_delete')
        g.custom_command('create', 'create_asset_track',
                         custom_command_type=get_custom_sdk('tracks', get_asset_tracks_client),
                         supports_no_wait=True)
        g.custom_command('update', 'update_track',
                         custom_command_type=get_custom_sdk('tracks', get_asset_tracks_client),
                         supports_no_wait=True)
        g.custom_command('update-data', 'update_track_data',
                         custom_command_type=get_custom_sdk('tracks', get_asset_tracks_client),
                         supports_no_wait=True)

    with self.command_group('ams job', get_sdk('Jobs', get_jobs_client)) as g:
        g.custom_show_command('show', 'get_job',
                              custom_command_type=get_custom_sdk('job', get_jobs_client))
        g.command('list', 'list')
        g.command('delete', 'delete')
        g.custom_command('cancel', 'cancel_job',
                         custom_command_type=get_custom_sdk('job', get_jobs_client))
        g.custom_command('start', 'create_job',
                         custom_command_type=get_custom_sdk('job', get_jobs_client))
        g.generic_update_command('update',
                                 setter_name='update',
                                 custom_func_name='update_job',
                                 custom_func_type=get_custom_sdk('job', get_jobs_client))

    with self.command_group('ams content-key-policy', get_sdk('ContentKeyPolicies', get_content_key_policies_client)) as g:
        g.custom_command('create', 'create_content_key_policy',
                         custom_command_type=get_custom_sdk('content_key_policy', get_content_key_policies_client))
        g.custom_show_command('show', 'show_content_key_policy',
                              custom_command_type=get_custom_sdk('content_key_policy', get_content_key_policies_client))
        g.command('delete', 'delete')
        g.command('list', 'list')
        g.generic_update_command('update',
                                 getter_name='get_policy_properties_with_secrets',
                                 setter_name='update_content_key_policy_setter',
                                 setter_type=get_custom_sdk('content_key_policy', get_content_key_policies_client),
                                 custom_func_name='update_content_key_policy',
                                 custom_func_type=get_custom_sdk('content_key_policy', get_content_key_policies_client))

    with self.command_group('ams content-key-policy option', get_sdk('ContentKeyPolicies', get_content_key_policies_client)) as g:
        g.custom_command('add', 'add_content_key_policy_option',
                         custom_command_type=get_custom_sdk('content_key_policy', get_content_key_policies_client))
        g.custom_command('remove', 'remove_content_key_policy_option',
                         custom_command_type=get_custom_sdk('content_key_policy', get_content_key_policies_client))
        g.custom_command('update', 'update_content_key_policy_option',
                         custom_command_type=get_custom_sdk('content_key_policy', get_content_key_policies_client))

    with self.command_group('ams streaming-locator', get_sdk('StreamingLocators', get_streaming_locators_client)) as g:
        g.custom_command('create', 'create_streaming_locator',
                         custom_command_type=get_custom_sdk('streaming_locator', get_streaming_locators_client))
        g.command('list', 'list')
        g.custom_show_command('show', 'get_streaming_locator',
                              custom_command_type=get_custom_sdk('streaming_locator', get_streaming_locators_client))
        g.command('delete', 'delete')
        g.command('get-paths', 'list_paths')
        g.custom_command('list-content-keys', 'list_content_keys',
                         custom_command_type=get_custom_sdk('streaming_locator', get_streaming_locators_client))

    with self.command_group('ams streaming-policy', get_sdk('StreamingPolicies', get_streaming_policies_client)) as g:
        g.custom_command('create', 'create_streaming_policy',
                         custom_command_type=get_custom_sdk('streaming_policy', get_streaming_policies_client))
        g.command('list', 'list')
        g.custom_show_command('show', 'get_streaming_policy',
                              custom_command_type=get_custom_sdk('streaming_policy', get_streaming_policies_client))
        g.command('delete', 'delete')

    with self.command_group('ams streaming-endpoint', get_sdk('StreamingEndpoints', get_streaming_endpoints_client)) as g:
        g.command('list', 'list')
        g.custom_command('start', 'start',
                         custom_command_type=get_custom_sdk('streaming_endpoint', get_streaming_endpoints_client),
                         supports_no_wait=True)
        g.custom_command('stop', 'stop',
                         custom_command_type=get_custom_sdk('streaming_endpoint', get_streaming_endpoints_client),
                         supports_no_wait=True)
        g.custom_command('create', 'create_streaming_endpoint',
                         custom_command_type=get_custom_sdk('streaming_endpoint', get_streaming_endpoints_client),
                         supports_no_wait=True)
        g.custom_command('scale', 'scale',
                         custom_command_type=get_custom_sdk('streaming_endpoint', get_streaming_endpoints_client))
        g.generic_update_command('update',
                                 setter_name='update_streaming_endpoint_setter',
                                 setter_type=get_custom_sdk('streaming_endpoint', get_streaming_endpoints_client),
                                 custom_func_name='update_streaming_endpoint',
                                 custom_func_type=get_custom_sdk('streaming_endpoint', get_streaming_endpoints_client),
                                 supports_no_wait=True)
        g.custom_show_command('show', 'get_streaming_endpoint',
                              custom_command_type=get_custom_sdk('streaming_endpoint', get_streaming_endpoints_client))
        g.command('delete', 'begin_delete')
        g.wait_command('wait')
        g.custom_command('get-skus', 'get_skus',
                         custom_command_type=get_custom_sdk('streaming_endpoint', get_streaming_endpoints_client))

    with self.command_group('ams streaming-endpoint akamai', get_sdk('StreamingEndpoints', get_streaming_endpoints_client)) as g:
        g.custom_command('add', 'add_akamai_access_control',
                         custom_command_type=get_custom_sdk('streaming_endpoint', get_streaming_endpoints_client))
        g.custom_command('remove', 'remove_akamai_access_control',
                         custom_command_type=get_custom_sdk('streaming_endpoint', get_streaming_endpoints_client))

    with self.command_group('ams live-event', get_sdk('LiveEvents', get_live_events_client)) as g:
        g.custom_command('create', 'create',
                         custom_command_type=get_custom_sdk('live_event', get_live_events_client),
                         supports_no_wait=True)
        g.custom_command('start', 'start',
                         custom_command_type=get_custom_sdk('live_event', get_live_events_client),
                         supports_no_wait=True)
        g.custom_command('standby', 'standby',
                         custom_command_type=get_custom_sdk('live_event', get_live_events_client),
                         supports_no_wait=True)
        g.custom_command('stop', 'stop',
                         custom_command_type=get_custom_sdk('live_event', get_live_events_client),
                         supports_no_wait=True)
        g.custom_command('reset', 'reset',
                         custom_command_type=get_custom_sdk('live_event', get_live_events_client),
                         supports_no_wait=True)
        g.custom_show_command('show', 'get_live_event',
                              custom_command_type=get_custom_sdk('live_event', get_live_events_client))
        g.command('delete', 'begin_delete')
        g.command('list', 'list')
        g.generic_update_command('update',
                                 setter_name='update_live_event_setter',
                                 setter_type=get_custom_sdk('live_event', get_live_events_client),
                                 custom_func_name='update_live_event',
                                 custom_func_type=get_custom_sdk('live_event', get_live_events_client))
        g.wait_command('wait')

    with self.command_group('ams live-output', get_sdk('LiveOutputs', get_live_outputs_client)) as g:
        g.custom_command('create', 'create_live_output',
                         custom_command_type=get_custom_sdk('live_output', get_live_outputs_client))
        g.custom_show_command('show', 'get_live_output',
                              custom_command_type=get_custom_sdk('live_output', get_live_outputs_client))
        g.command('list', 'list')
        g.command('delete', 'begin_delete')

    with self.command_group('ams account-filter', get_sdk('AccountFilters', get_account_filters_client)) as g:
        g.custom_command('create', 'create_account_filter',
                         custom_command_type=get_custom_sdk('account_filter', get_account_filters_client))
        g.custom_show_command('show', 'get_account_filter',
                              custom_command_type=get_custom_sdk('account_filter', get_account_filters_client))
        g.command('list', 'list')
        g.command('delete', 'delete')
        g.generic_update_command('update',
                                 custom_func_name='update_account_filter',
                                 custom_func_type=get_custom_sdk('account_filter', get_mediaservices_client))

    with self.command_group('ams'):
        pass
